#define _RETAIL
#define WIN32_LEAN_AND_MEAN
#include <GMLuaModule.h>
#include "gm_clientutils.h"

#include <eiface.h>
class CBaseHandle;
#include <usercmd.h>
#include <cdll_int.h>

IVEngineClient *engine = NULL;

LUA_FUNCTION(GetUcmdButtons)
{
	ILuaInterface *gLua = Lua();

	gLua->CheckType(1, GLua::TYPE_USERCMD);

	CUserCmd *cmd = (CUserCmd *)gLua->GetUserData(1);

	gLua->PushLong(cmd->buttons);

	return 1;
}

LUA_FUNCTION(SetUcmdButtons)
{
	ILuaInterface *gLua = Lua();

	gLua->CheckType(1, GLua::TYPE_USERCMD);
	gLua->CheckType(2, GLua::TYPE_NUMBER);

	CUserCmd *cmd = (CUserCmd *)gLua->GetUserData(1);
	int buttons = gLua->GetInteger(2);

	cmd->buttons = buttons;

	return 0;
}

LUA_FUNCTION(SetImpulse)
{
	ILuaInterface *gLua = Lua();

	gLua->CheckType(1, GLua::TYPE_USERCMD);
	gLua->CheckType(2, GLua::TYPE_NUMBER);

	CUserCmd *cmd = (CUserCmd *)gLua->GetUserData(1);
	int impulse = gLua->GetInteger(2);

	if(impulse > 255 || impulse < 0)
	{
		gLua->Error("Impulse out of range!\n");
		return 0;
	}

	cmd->impulse = impulse;

	return 0;
}

// this is not completely accurate, because a bool is only a bit
LUA_FUNCTION(GetUmsgSize)
{
	ILuaInterface *gLua = Lua();

	gLua->CheckType(1, GLua::TYPE_USERMSG);

	bf_read *buf = (bf_read *)gLua->GetUserData( 1 );

	gLua->PushLong( buf->TotalBytesAvailable() );
	return 1;
}

LUA_FUNCTION(GetKeyForBinding)
{
	ILuaInterface *gLua = Lua();

	gLua->CheckType(1, GLua::TYPE_STRING);

	const char *key = engine->Key_LookupBinding(gLua->GetString(1));

	if(!key)
		return 0;

	gLua->Push(key);
	return 1;
}

int Start(lua_State *L)
{
	ILuaInterface *gLua = Lua();

	CreateInterfaceFn engineFactory = Sys_GetFactory("engine.dll");

	engine = (IVEngineClient *)engineFactory(VENGINE_CLIENT_INTERFACE_VERSION, NULL);

	if(!engine)
	{
		gLua->Error("Could not load engine interface.\n");
		return 0;
	}

	gLua->SetGlobal("GetUcmdButtons", GetUcmdButtons);
	gLua->SetGlobal("SetUcmdButtons", SetUcmdButtons);
	gLua->SetGlobal("SetImpulse", SetImpulse);

	gLua->SetGlobal("GetUmsgSize", GetUmsgSize);

	gLua->SetGlobal("GetKeyForBinding", GetKeyForBinding);

	return 0;
}

int Close(lua_State *L)
{
	return 0;
}